/*-
 *
 * Thu Nov 25 16:57:50 EET 1993
 */

#include "param.h"
#include "systm.h"
#include "malloc.h"
#include "stat.h"
#include "ioctl.h"
#include "socket.h"
#include "socketvar.h"
#include "protosw.h"
#include "domain.h"
#include "mbuf.h"

#include "net/if.h"
#include "net/route.h"
#include "net/af.h"
#include "net/raw_cb.h"

#include "ax25/ax.h"
#include "ax25/ax25.h"
#include "ax25/ax_pcb.h"
#include "ax25/ax_var.h"
#include "ax25/ax_raw.h"

#define sotoaxrawcb(so) ((struct axrawcb *) (so)->so_pcb)

static struct sockaddr_ax axdst = { sizeof(axdst), AF_AX25 };
static struct sockaddr_ax axsrc = { sizeof(axsrc), AF_AX25 };
static struct sockproto axproto = { PF_AX25 };

axraw_input(ifp, pid, m)
	struct ifnet *ifp;
	u_char pid;
	struct mbuf *m;
{
	struct axhdr *axh = mtod(m, struct axhdr *);

	axdst.sax_addr = axh->ah_dst;
	axdst.sax_addr.ax_ssid &= SSID;
	axsrc.sax_addr = axh->ah_src;
	axsrc.sax_addr.ax_ssid &= SSID;

	axproto.sp_protocol = pid;

	if (raw_input(m, &axproto, &axsrc, &axdst) == 0) {
		axstat.axs_noproto++;
	}
}

axraw_output(m, so)
	register struct mbuf *m;
	struct socket *so;
{
	register struct axrawcb *axr = sotoaxrawcb(so);

	if (axr->axr_ifp == NULL) {
		if (ax_ifaddr) {
			/*
			 * Pickup first configured if
			 */
			axr->axr_ifp = ax_ifaddr->aa_ifp;
		} else {
			m_freem(m);
			return (ENETDOWN);
		}
	}
	if (axr->axr_faddr && axr->axr_laddr) {

		return (ax25_send(axr->axr_ifp,
			&((struct sockaddr_ax *) axr->axr_faddr)->sax_addr,
			&((struct sockaddr_ax *) axr->axr_laddr)->sax_addr,
			COMMAND, UI, axr->axr_proto.sp_protocol, m));
	}
	if (m->m_pkthdr.len < AXMINLEN) {
		m_freem(m);
		return (EMSGSIZE);
	}
	return (ax25_sendraw(axr->axr_ifp, m));
}

/*ARGSUSED*/
axraw_usrreq(so, req, m, nam, control)
	register struct socket *so;
	int req;
	struct mbuf *m, *nam, *control;
{
	struct sockaddr_ax *sax;
	register struct axrawcb *axr = sotoaxrawcb(so);
	int error;

	switch (req) {

	case PRU_ATTACH:
		if (axr)
			panic("axraw_usrreq already attached");
		MALLOC(axr, struct axrawcb *, sizeof *axr, M_PCB, M_NOWAIT);
		if (axr == 0)
			return (ENOBUFS);
		bzero((caddr_t) axr, sizeof(*axr));
		so->so_pcb = (caddr_t) axr;
		error = raw_usrreq(so, req, m, nam, control);
		if (error) {
			if (so->so_pcb)
				free(so->so_pcb, M_PCB);
			return (error);
		}
		return (0);

	case PRU_BIND:
	case PRU_CONNECT:
		sax = mtod(nam, struct sockaddr_ax *);

		if (nam->m_len != sizeof(*sax) ||
			sax->sax_len != sizeof(*sax))
			return (EINVAL);
		if (sax->sax_family != AF_AX25)
			return (EAFNOSUPPORT);

		if (sax->sax_ifname[0]) {
			axr->axr_ifp = ax_iflookup(sax->sax_ifname);
			if (axr->axr_ifp == NULL)
				return (EADDRNOTAVAIL);
		}
		if (req == PRU_CONNECT) {
			axr->axr_remote = *sax;
			axr->axr_faddr = (struct sockaddr *) &axr->axr_remote;
			soisconnected(so);
		} else {
			axr->axr_local = *sax;
			axr->axr_laddr = (struct sockaddr *) &axr->axr_local;
		}
		return (0);

	case PRU_DETACH:
		if (axr == NULL)
			panic("axraw_usrreq already detached");
		axr->axr_laddr = axr->axr_faddr = NULL;
		break;
	}
	return (raw_usrreq(so, req, m, nam, control));
}

axraw_ctloutput(op, so, level, optname, m)
	int op;
	struct socket *so;
	int level, optname;
	struct mbuf **m;
{
	int error = 0;
	register struct axrawcb *axr = sotoaxrawcb(so);

	switch (op) {

	case PRCO_SETOPT:
		switch (optname) {

		default:
			error = EINVAL;
			break;
		}
		break;

	case PRCO_GETOPT:
		*m = m_get(M_WAIT, MT_SOOPTS);
		switch (optname) {

		default:
			error = EINVAL;
			m_freem(*m);
			*m = 0;
			break;
		}
		break;
	}
	if (op == PRCO_SETOPT && *m)
		(void) m_free(*m);
	return (error);
}

