/*-
 * Copyright (c) 1982, 1989 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)if_axsubr.c			0.0 (Skinnarila) 20/10/93
 */

#include "param.h"
#include "systm.h"
#include "kernel.h"
#include "malloc.h"
#include "mbuf.h"
#include "protosw.h"
#include "socket.h"
#include "ioctl.h"
#include "errno.h"
#include "syslog.h"

#include "net/if.h"
#include "net/netisr.h"
#include "net/route.h"
#include "net/if_llc.h"
#include "net/if_dl.h"

#ifdef INET
#include "../netinet/in.h"
#include "../netinet/in_var.h"
#include "../netinet/in_arp.h"
#endif
#include "ax25/ax.h"
#include "ax25/ax25.h"
#include "ax25/ax_var.h"

/*
 * AX25 output routine.
 */
ax_output(ifp, m0, dst, rt)
	register struct ifnet *ifp;
	struct mbuf *m0;
	struct sockaddr *dst;
	struct rtentry *rt;
{
#define	ac ((struct arpcom *) ifp)
	struct in_addr idst;
 	struct ax_addr axdstbuf, *axdst;
	struct axhdr *hdr;
	u_char pid;
	register struct mbuf *m = m0;
	struct mbuf *mcopy = NULL;
	int usetrailers, len = m->m_pkthdr.len;

	switch (dst->sa_family) {

#ifdef INET
	case AF_INET:
 		axdst = &axdstbuf;
		idst = ((struct sockaddr_in *)dst)->sin_addr;
 		if (!arpresolve(ac, m, &idst, axdst, &usetrailers))
			return (0);	/* if not yet resolved */

		if ((ifp->if_flags & IFF_SIMPLEX) && ax_ismulticast(axdst))
			mcopy = m_copy(m, 0, (int)M_COPYALL);

		pid = AXTYPE_IP;
		break;
#endif

	case AF_UNSPEC:
		axdst = (struct ax_addr *) dst->sa_data;
		pid = dst->sa_data[AXALEN];
		break;

	default:
		printf("%s%d: can't handle af%d\n",
			ifp->if_name, ifp->if_unit, dst->sa_family);
		if (m)
			m_freem(m);
		return (EAFNOSUPPORT);
	}
	if (mcopy)
		(void) looutput(ifp, mcopy, dst, rt);

	M_PREPEND(m, 2 * AXALEN + 2, M_DONTWAIT);
	if (m == NULL)
		return (ENOBUFS);

	hdr = mtod(m, struct axhdr *);

	hdr->ah_dst = *axdst;
	hdr->ah_src = *(struct ax_addr *) ac->ac_hwaddr;

	hdr->ah_dst.ax_ssid &= (SSID);
	hdr->ah_dst.ax_ssid |= (SSID_C | SSID_XXX);
	hdr->ah_src.ax_ssid &= (SSID);
	hdr->ah_src.ax_ssid |= (SSID_XXX | SSID_E);

	hdr->ah_ctrl = UI;
	hdr->ah_pid = pid;

	return (ax25_sendraw(ifp, m));
}

/*
 * Generic AX25 send routine.
 */
ax25_send(ifp, dst, src, cmdrsp, ctrl, pid, m)
	struct ifnet *ifp;
	struct ax_addr *dst, *src;
	int cmdrsp;
	u_char ctrl, pid;
	struct mbuf *m;
{
	struct axhdr *hdr;
	int s, len, hlen = 2 * AXALEN + 1;

	if ((ifp->if_flags & (IFF_UP|IFF_RUNNING)) != (IFF_UP|IFF_RUNNING)) {
		if (m)
			m_freem(m);
		return (ENETDOWN);
	}
	if (pid)
		++hlen;

	if (m) {
		len = m->m_pkthdr.len;
		M_PREPEND(m, hlen, M_DONTWAIT);
	} else {
		len = 0;
		MGETHDR(m, M_DONTWAIT, MT_HEADER);
		if (m) {
			m->m_pkthdr.len = hlen;
			m->m_len = hlen;
			m->m_next = NULL;
		}
	}
	if (m == NULL)
		return (ENOBUFS);

	hdr = mtod(m, struct axhdr *);

	hdr->ah_dst = *dst;
	hdr->ah_src = *src;
	if (cmdrsp == COMMAND) {
		hdr->ah_dst.ax_ssid &= (SSID);
		hdr->ah_dst.ax_ssid |= (SSID_C | SSID_XXX);
		hdr->ah_src.ax_ssid &= (SSID);
		hdr->ah_src.ax_ssid |= (SSID_XXX | SSID_E);
	} else {
		hdr->ah_dst.ax_ssid &= (SSID);
		hdr->ah_dst.ax_ssid |= (SSID_XXX);
		hdr->ah_src.ax_ssid &= (SSID);
		hdr->ah_src.ax_ssid |= (SSID_C | SSID_XXX | SSID_E);
	}
	hdr->ah_ctrl = ctrl;
	if (pid)
		hdr->ah_pid = pid;

	if (ax_debug) {
		ax_trace("axo", m);
	}
	{
		extern int ax_monitored;
		if (ax_monitored)
			ax_monitor("axo", m);
	}
	if (ax_ismulticast(dst))
		ifp->if_omcasts++;

	s = splimp();
	/*
	 * Queue message on interface,
	 * and start output if interface not yet active.
	 */
	ifp->if_lastchange = time;

	if (IF_QFULL(&ifp->if_snd)) {
		IF_DROP(&ifp->if_snd);
		splx(s);
		if (m)
			m_freem(m);
		return (ENOBUFS);
	}
	IF_ENQUEUE(&ifp->if_snd, m);
	if ((ifp->if_flags & IFF_OACTIVE) == 0)
		(*ifp->if_start)(ifp);
	splx(s);

	ifp->if_obytes += len;

	return (0);
}

/*
 * Raw AX25 send routine.
 */
ax25_sendraw(ifp, m)
	struct ifnet *ifp;
	struct mbuf *m;
{
	int s, len = m->m_pkthdr.len;

	if ((ifp->if_flags & (IFF_UP|IFF_RUNNING)) != (IFF_UP|IFF_RUNNING)) {
		if (m)
			m_freem(m);
		return (ENETDOWN);
	}
	if (ax_debug) {
		ax_trace("axo", m);
	}
	{
		extern int ax_monitored;
		if (ax_monitored)
			ax_monitor("axo", m);
	}
	s = splimp();
	/*
	 * Queue message on interface,
	 * and start output if interface not yet active.
	 */
	ifp->if_lastchange = time;

	if (IF_QFULL(&ifp->if_snd)) {
		IF_DROP(&ifp->if_snd);
		splx(s);
		if (m)
			m_freem(m);
		return (ENOBUFS);
	}
	IF_ENQUEUE(&ifp->if_snd, m);
	if ((ifp->if_flags & IFF_OACTIVE) == 0)
		(*ifp->if_start)(ifp);
	splx(s);
	ifp->if_obytes += len;

	return (0);
}

